@@ -11,6 +11,8 @@ from ..hashfile.meta import Meta
 from ..hashfile.obj import HashFile
 
 if TYPE_CHECKING:
+    from pygtrie import Trie
+
     from ..hashfile.db import HashFileDB
     from ..hashfile.hash_info import HashInfo
 
@@ -50,24 +52,27 @@ class Tree(HashFile):
         self.hash_info = None
         self.oid = None
         self._dict: Dict[
-            Tuple[str, ...], Tuple[Optional["Meta"], "HashInfo"]
+            Tuple[str, ...], Tuple[Optional["Meta"], Optional["HashInfo"]]
         ] = {}
 
     @cached_property
-    def _trie(self):
+    def _trie(self) -> "Trie":
         from pygtrie import Trie
 
         return Trie(self._dict)
 
     def add(
-        self, key: Tuple[str, ...], meta: Optional["Meta"], oid: "HashInfo"
+        self,
+        key: Tuple[str, ...],
+        meta: Optional["Meta"],
+        oid: Optional["HashInfo"],
     ):
         self.__dict__.pop("_trie", None)
         self._dict[key] = (meta, oid)
 
     def get(
         self, key: Tuple[str, ...], default=None
-    ) -> Optional[Tuple[Optional["Meta"], "HashInfo"]]:
+    ) -> Optional[Tuple[Optional["Meta"], Optional["HashInfo"]]]:
         return self._dict.get(key, default)
 
     def digest(self, with_meta: bool = False):
@@ -131,7 +136,7 @@ class Tree(HashFile):
             (
                 {
                     **(meta.to_dict() if with_meta else {}),
-                    **hi.to_dict(),
+                    **(hi.to_dict() if hi else {}),
                     self.PARAM_RELPATH: posixpath.sep.join(parts),
                 }
                 for parts, meta, hi in self  # noqa: B301
@@ -139,6 +144,9 @@ class Tree(HashFile):
             key=itemgetter(self.PARAM_RELPATH),
         )
 
+    def as_trie(self) -> "Trie":
+        return self._trie.copy()
+
     def as_bytes(self, with_meta: bool = False):
         return json.dumps(
             self.as_list(with_meta=with_meta), sort_keys=True
@@ -161,6 +169,12 @@ class Tree(HashFile):
             tree.add(parts, meta, hash_info)
         return tree
 
+    @classmethod
+    def from_trie(cls, trie: "Trie") -> "Tree":
+        tree = cls()
+        tree._dict = dict(trie.iteritems())
+        return tree
+
     @classmethod
     def load(cls, odb, hash_info, hash_name: Optional[str] = None) -> "Tree":
         obj = odb.get(hash_info.value)
