@@ -102,6 +102,13 @@ def _diff_entry(
     meta_diff = _diff_meta(old_meta, new_meta, cmp_key=meta_cmp_key)
     hi_diff = _diff_hash_info(old_hi, new_hi)
 
+    if old is None and new is not None:
+        entry_diff = ADD
+    elif old is not None and new is None:
+        entry_diff = DELETE
+    else:
+        entry_diff = UNCHANGED
+
     if meta_only:
         return meta_diff
 
@@ -118,7 +125,7 @@ def _diff_entry(
 
     # Only return UNCHANGED/ADD/DELETE when hi_diff and meta_diff match,
     # otherwise return MODIFY
-    if meta_diff == hi_diff:
+    if meta_diff == hi_diff == entry_diff:
         return meta_diff
 
     return MODIFY
@@ -136,7 +143,7 @@ def _get_items(
     unknown = False
 
     try:
-        if index is not None and not (shallow and entry):
+        if index is not None and not (shallow and entry and entry.hash_info):
             items = dict(index.ls(key, detail=True))
     except KeyError:
         pass
@@ -157,19 +164,24 @@ def _diff(
     meta_cmp_key: Optional[Callable[["Meta"], Any]] = None,
     shallow: Optional[bool] = False,
 ):
-    todo = deque([((), None, None, False)])
+    old_root_items = {}
+    new_root_items = {}
+
+    if old is not None:
+        try:
+            old_root_items[()] = old.info(())
+        except FileNotFoundError:
+            pass
+
+    if new is not None:
+        try:
+            new_root_items[()] = new.info(())
+        except FileNotFoundError:
+            pass
+
+    todo = deque([(old_root_items, new_root_items, False)])
     while todo:
-        dirkey, old_direntry, new_direntry, unknown = todo.popleft()
-
-        kwargs = {"shallow": shallow, "with_unknown": with_unknown}
-        old_items, old_unknown = _get_items(
-            old, dirkey, old_direntry, **kwargs
-        )
-        new_items, new_unknown = _get_items(
-            new, dirkey, new_direntry, **kwargs
-        )
-        unknown = old_unknown or new_unknown
-
+        old_items, new_items, unknown = todo.popleft()
         for key in old_items.keys() | new_items.keys():
             old_info = old_items.get(key) or {}
             new_info = new_items.get(key) or {}
@@ -201,7 +213,15 @@ def _diff(
                 old_info.get("type") == "directory"
                 or new_info.get("type") == "directory"
             ):
-                todo.append((key, old_entry, new_entry, unknown))
+                kwargs = {"shallow": shallow, "with_unknown": with_unknown}
+                old_dir_items, old_unknown = _get_items(
+                    old, key, old_entry, **kwargs
+                )
+                new_dir_items, new_unknown = _get_items(
+                    new, key, new_entry, **kwargs
+                )
+                dir_unknown = old_unknown or new_unknown
+                todo.append((old_dir_items, new_dir_items, dir_unknown))
 
             if old_entry is None and new_entry is None:
                 continue
