@@ -1,5 +1,6 @@
+import errno
+import os
 from abc import ABC, abstractmethod
-from collections import defaultdict
 from dataclasses import dataclass
 from functools import cached_property
 from typing import (
@@ -111,6 +112,20 @@ class Storage(ABC):
     def __init__(self, key: "DataIndexKey"):
         self.key = key
 
+    @property
+    @abstractmethod
+    def fs(self):
+        pass
+
+    @property
+    @abstractmethod
+    def path(self):
+        pass
+
+    @abstractmethod
+    def get_key(self, entry: "DataIndexEntry") -> "DataIndexKey":
+        pass
+
     @abstractmethod
     def get(self, entry: "DataIndexEntry") -> Tuple["FileSystem", str]:
         pass
@@ -131,6 +146,21 @@ class ObjectStorage(Storage):
         self.index = index
         super().__init__(key)
 
+    @property
+    def fs(self):
+        return self.odb.fs
+
+    @property
+    def path(self):
+        return self.odb.path
+
+    def get_key(self, entry: "DataIndexEntry") -> "DataIndexKey":
+        assert entry.hash_info
+        assert entry.hash_info.value
+        return self.odb._oid_parts(  # pylint: disable=protected-access
+            entry.hash_info.value
+        )
+
     def get(self, entry: "DataIndexEntry") -> Tuple["FileSystem", str]:
         if not entry.hash_info:
             raise ValueError
@@ -171,11 +201,29 @@ class FileStorage(Storage):
         path: "str",
         index: Optional["DataIndex"] = None,
     ):
-        self.fs = fs
-        self.path = path
+        self._fs = fs
+        self._path = path
         self.index = index
         super().__init__(key)
 
+    @property
+    def fs(self):
+        return self._fs
+
+    @property
+    def path(self):
+        return self._path
+
+    def get_key(self, entry: "DataIndexEntry") -> "DataIndexKey":
+        assert entry.key
+        key: "DataIndexKey" = entry.key[len(self.key) :]
+        if self.fs.version_aware and entry.meta and entry.meta.version_id:
+            key = (
+                *key[:-1],
+                self.fs.path.version_path(key[-1], entry.meta.version_id),
+            )
+        return key
+
     def get(self, entry: "DataIndexEntry") -> Tuple["FileSystem", str]:
         assert entry.key is not None
         path = self.fs.path.join(self.path, *entry.key[len(self.key) :])
@@ -377,8 +425,6 @@ class BaseDataIndex(ABC, MutableMapping[DataIndexKey, DataIndexEntry]):
                 "type": "directory",
                 "size": 0,
                 "isexec": False,
-                "isdvc": bool(self.longest_prefix(key)),
-                "isout": False,
                 "entry": None,
             }
 
@@ -392,8 +438,6 @@ class BaseDataIndex(ABC, MutableMapping[DataIndexKey, DataIndexEntry]):
             "type": "directory" if isdir else "file",
             "size": meta.size if meta else 0,
             "isexec": meta.isexec if meta else False,
-            "isdvc": True,
-            "isout": True,
             "entry": entry,
         }
 
@@ -422,6 +466,9 @@ class BaseDataIndex(ABC, MutableMapping[DataIndexKey, DataIndexEntry]):
 
 
 def _load_from_object_storage(trie, root_entry, storage):
+    if not root_entry.hash_info or not root_entry.hash_info.isdir:
+        raise FileNotFoundError
+
     obj = Tree.load(storage.odb, root_entry.hash_info, hash_name="md5")
 
     dirs = set()
@@ -461,6 +508,9 @@ def _load_from_file_storage(trie, root_entry, storage):
 
     fs, path = storage.get(root_entry)
 
+    if not fs.exists(path):
+        raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), path)
+
     for entry in build_entries(path, fs):
         entry.key = root_entry.key + entry.key
         trie[entry.key] = entry
@@ -480,11 +530,7 @@ def _load_from_storage(trie, entry, storage_info):
             continue
 
         try:
-            if (
-                isinstance(storage, ObjectStorage)
-                and entry.hash_info
-                and entry.hash_info.isdir
-            ):
+            if isinstance(storage, ObjectStorage):
                 _load_from_object_storage(trie, entry, storage)
             else:
                 _load_from_file_storage(trie, entry, storage)
@@ -631,30 +677,3 @@ class DataIndex(BaseDataIndex, MutableMapping[DataIndexKey, DataIndexEntry]):
             )
         else:
             yield from self._trie.ls(root_key)
-
-
-def transfer(index, src, dst):
-    from ..hashfile.transfer import transfer as otransfer
-
-    by_direction = defaultdict(set)
-    for _, entry in index.iteritems():
-        src_odb = getattr(entry, src)
-        assert src_odb
-        dst_odb = getattr(entry, dst)
-        assert dst_odb
-        by_direction[(src_odb, dst_odb)].add(entry.hash_info)
-
-    for (src_odb, dst_odb), hash_infos in by_direction.items():
-        otransfer(src_odb, dst_odb, hash_infos)
-
-
-def commit(index):
-    transfer(index, "odb", "cache")
-
-
-def push(index):
-    transfer(index, "cache", "remote")
-
-
-def fetch(index):
-    transfer(index, "remote", "cache")
